@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,27 +49,34 @@
 
 package com.openexchange.calendar;
 
+import static com.openexchange.java.Autoboxing.I;
 import java.sql.Connection;
 import java.sql.SQLException;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Date;
 import java.util.List;
-
+import java.util.Map;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
-
 import com.openexchange.api.OXPermissionException;
 import com.openexchange.api2.OXException;
+import com.openexchange.calendar.storage.ParticipantStorage;
 import com.openexchange.configuration.ServerConfig;
 import com.openexchange.configuration.ServerConfig.Property;
 import com.openexchange.database.DBPoolingException;
 import com.openexchange.groupware.AbstractOXException;
+import com.openexchange.groupware.Types;
+import com.openexchange.groupware.attach.AttachmentBase;
+import com.openexchange.groupware.attach.AttachmentException;
+import com.openexchange.groupware.attach.Attachments;
 import com.openexchange.groupware.calendar.CalendarConfig;
 import com.openexchange.groupware.calendar.CalendarDataObject;
 import com.openexchange.groupware.calendar.OXCalendarException;
 import com.openexchange.groupware.container.FolderObject;
 import com.openexchange.groupware.container.UserParticipant;
 import com.openexchange.groupware.contexts.Context;
+import com.openexchange.groupware.tx.SimpleDBProvider;
 import com.openexchange.server.impl.DBPool;
 import com.openexchange.tools.StringCollection;
 import com.openexchange.tools.iterator.SearchIterator;
@@ -82,6 +89,8 @@ import com.openexchange.tools.iterator.SearchIteratorException;
 
 public class CachedCalendarIterator implements SearchIterator<CalendarDataObject> {
 
+    private static final Log LOG = LogFactory.getLog(CachedCalendarIterator.class);
+
 	private final List<AbstractOXException> warnings;
     private final List<CalendarDataObject> list;
     private final SearchIterator<CalendarDataObject> non_cached_iterator;
@@ -93,13 +102,11 @@ public class CachedCalendarIterator implements SearchIterator<CalendarDataObject
     private int[][] oids;
     private int cc;
     private boolean oxonfe;
-    
+
     public static boolean CACHED_ITERATOR_FAST_FETCH = true;
     public static int MAX_PRE_FETCH = 20;
     private int pre_fetch;
-    
-    private static final Log LOG = LogFactory.getLog(CachedCalendarIterator.class);
-    
+
     public CachedCalendarIterator(final SearchIterator<CalendarDataObject> non_cached_iterator, final Context c, final int uid) throws SearchIteratorException, OXException {
     	this.warnings =  new ArrayList<AbstractOXException>(2);
     	list = new ArrayList<CalendarDataObject>(16);
@@ -113,7 +120,7 @@ public class CachedCalendarIterator implements SearchIterator<CalendarDataObject
             fillCachedResultSet();
         }
     }
-    
+
     public CachedCalendarIterator(final SearchIterator<CalendarDataObject> non_cached_iterator, final Context c, final int uid, final int[][] oids) throws SearchIteratorException, OXException {
     	this.warnings =  new ArrayList<AbstractOXException>(2);
     	if (non_cached_iterator.hasWarnings()) {
@@ -131,7 +138,7 @@ public class CachedCalendarIterator implements SearchIterator<CalendarDataObject
             fillCachedResultSet();
         }
     }
-    
+
     public boolean hasNext() {
         if (!cache) {
             return non_cached_iterator.hasNext();
@@ -145,7 +152,7 @@ public class CachedCalendarIterator implements SearchIterator<CalendarDataObject
 		}
         return false;
     }
-    
+
     public CalendarDataObject next() throws SearchIteratorException, OXException {
         if (!oxonfe) {
             if (!cache) {
@@ -169,7 +176,7 @@ public class CachedCalendarIterator implements SearchIterator<CalendarDataObject
         }
         return null;
     }
-    
+
     public final void close() throws SearchIteratorException {
         if (closed) {
             return;
@@ -177,11 +184,11 @@ public class CachedCalendarIterator implements SearchIterator<CalendarDataObject
         closed = true;
         non_cached_iterator.close();
     }
-    
+
     public int size() {
         return non_cached_iterator.size();
     }
-    
+
     public boolean hasSize() {
         return non_cached_iterator.hasSize();
     }
@@ -197,7 +204,7 @@ public class CachedCalendarIterator implements SearchIterator<CalendarDataObject
 	public boolean hasWarnings() {
 		return !warnings.isEmpty();
 	}
-    
+
     private final void fillCachedResultSet() throws SearchIteratorException, OXException {
         try {
             while (non_cached_iterator.hasNext()) {
@@ -207,56 +214,61 @@ public class CachedCalendarIterator implements SearchIterator<CalendarDataObject
             close();
         }
     }
-    
+
     private final CalendarDataObject getPreFilledResult() throws SearchIteratorException, OXException {
         Connection readcon = null;
         CalendarDataObject cdao;
-        CalendarDataObject temp;
         try {
             cdao = list.get(counter++);
             if (CACHED_ITERATOR_FAST_FETCH && pre_fetch < counter) {
-			    if ((cdao.fillParticipants() || cdao.fillUserParticipants() || (cdao.fillFolderID()))) {
+			    if ((cdao.fillParticipants() || cdao.fillUserParticipants() || cdao.fillFolderID() || cdao.fillConfirmations())) {
 			        try {
 			            readcon = DBPool.pickup(c);
 			        } catch (final DBPoolingException ex) {
 			            throw new OXException(ex);
 			        }
 			    }
-			    
+			
 			    int mn = MAX_PRE_FETCH;
 			    if (mn+pre_fetch > list.size()) {
 			        mn = list.size()%MAX_PRE_FETCH;
 			    }
 			    final int arr[] = new int[mn];
-			    
+			
 			    for (int a = 0; a < mn; a++) {
-			        temp = list.get(pre_fetch++);
+			        CalendarDataObject temp = list.get(pre_fetch++);
 			        arr[a] = temp.getObjectID();
 			    }
-			    
+			
 			    final String sqlin = StringCollection.getSqlInString(arr);
 			    if  (cdao.fillUserParticipants() || cdao.fillFolderID()) {
 			        try {
 			            new CalendarMySQL().getUserParticipantsSQLIn(list, readcon, c.getContextId(), uid, sqlin);
 			        } catch (final SQLException ex) {
-			            throw new OXCalendarException(OXCalendarException.Code.UNEXPECTED_EXCEPTION, ex, Integer.valueOf(202));
+			            throw new OXCalendarException(OXCalendarException.Code.UNEXPECTED_EXCEPTION, ex, I(202));
 			        }
 			    }
-			    
+			
 			    if (cdao.fillParticipants()) {
 			        try {
 			            new CalendarMySQL().getParticipantsSQLIn(list, readcon, cdao.getContextID(), sqlin);
 			        } catch (final SQLException ex) {
-			            throw new OXCalendarException(OXCalendarException.Code.UNEXPECTED_EXCEPTION, ex, Integer.valueOf(203));
+			            throw new OXCalendarException(OXCalendarException.Code.UNEXPECTED_EXCEPTION, ex, I(203));
 			        }
 			    }
+			    if (cdao.fillConfirmations()) {
+			        ParticipantStorage.getInstance().selectExternal(c, readcon, list, arr);
+			    }
+			    if (cdao.isFillLastModifiedOfNewestAttachment()) {
+			        setAttachmentLastModified(c, readcon, list, arr);
+			    }
 			}
-            
+
             if (cdao.fillFolderID()) {
                 cdao.setGlobalFolderID(cdao.getEffectiveFolderId());
             }
-            
-            // Security check for bug 10836 
+
+            // Security check for bug 10836
             if (CACHED_ITERATOR_FAST_FETCH && (cdao.getFolderType() == FolderObject.PRIVATE || cdao.getFolderType() == FolderObject.SHARED)) {
             	final UserParticipant up[] = cdao.getUsers();
             	boolean found = false;
@@ -270,7 +282,7 @@ public class CachedCalendarIterator implements SearchIterator<CalendarDataObject
             		throw new OXPermissionException(new OXCalendarException(OXCalendarException.Code.LOAD_PERMISSION_EXCEPTION_5));
             	}
             }
-            
+
         } finally {
             if (readcon != null) {
                 DBPool.push(c, readcon);
@@ -279,5 +291,21 @@ public class CachedCalendarIterator implements SearchIterator<CalendarDataObject
         }
         return cdao;
     }
-    
+
+    private static void setAttachmentLastModified(Context ctx, Connection readcon, List<CalendarDataObject> list, int[] arr) {
+        AttachmentBase attachmentBase = Attachments.getInstance(new SimpleDBProvider(readcon, null));
+        final Map<Integer, Date> dates;
+        try {
+            dates = attachmentBase.getNewestCreationDates(ctx, Types.APPOINTMENT, arr);
+        } catch (AttachmentException e) {
+            LOG.error(e.getMessage(), e);
+            return;
+        }
+        for (CalendarDataObject cdao : list) {
+            Date date = dates.get(I(cdao.getObjectID()));
+            if (null != date) {
+                cdao.setLastModifiedOfNewestAttachment(date);
+            }
+        }
+    }
 }
