@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -50,6 +50,7 @@
 package com.openexchange.calendar;
 
 import static com.openexchange.groupware.EnumComponent.APPOINTMENT;
+import static com.openexchange.java.Autoboxing.I;
 import java.sql.Connection;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
@@ -68,21 +69,28 @@ import com.openexchange.api.OXObjectNotFoundException;
 import com.openexchange.api.OXPermissionException;
 import com.openexchange.api2.OXException;
 import com.openexchange.calendar.api.CalendarCollection;
+import com.openexchange.calendar.storage.ParticipantStorage;
 import com.openexchange.group.Group;
 import com.openexchange.group.GroupStorage;
 import com.openexchange.groupware.AbstractOXException;
+import com.openexchange.groupware.Types;
+import com.openexchange.groupware.attach.AttachmentBase;
+import com.openexchange.groupware.attach.AttachmentException;
+import com.openexchange.groupware.attach.Attachments;
 import com.openexchange.groupware.calendar.CalendarDataObject;
 import com.openexchange.groupware.calendar.Constants;
 import com.openexchange.groupware.calendar.OXCalendarException;
 import com.openexchange.groupware.calendar.RecurringResultInterface;
 import com.openexchange.groupware.calendar.RecurringResultsInterface;
 import com.openexchange.groupware.container.Appointment;
+import com.openexchange.groupware.container.ExternalUserParticipant;
 import com.openexchange.groupware.container.FolderObject;
 import com.openexchange.groupware.container.Participant;
 import com.openexchange.groupware.container.Participants;
 import com.openexchange.groupware.container.UserParticipant;
 import com.openexchange.groupware.contexts.Context;
 import com.openexchange.groupware.ldap.LdapException;
+import com.openexchange.groupware.tx.SimpleDBProvider;
 import com.openexchange.server.impl.DBPool;
 import com.openexchange.session.Session;
 import com.openexchange.tools.StringCollection;
@@ -176,7 +184,9 @@ public class CalendarOperation implements SearchIterator<CalendarDataObject> {
                 cdao.setStartDate(setDate(i++, load_resultset));
                 cdao.setEndDate(setDate(i++, load_resultset));
                 cdao.setTimezone(setString(i++, load_resultset));
-                cdao.setRecurrenceID(setInt(i++, load_resultset));
+                int recId = setInt(i++, load_resultset);
+                if (recId != 0)
+                    cdao.setRecurrenceID(recId);
                 cdao.setLabel(setInt(i++, load_resultset));
                 cdao.setTitle(setString(i++, load_resultset));
                 cdao.setLocation(setString(i++, load_resultset));
@@ -185,6 +195,11 @@ public class CalendarOperation implements SearchIterator<CalendarDataObject> {
                 cdao.setNote(setString(i++, load_resultset));
                 cdao.setFullTime(setBooleanToInt(i++, load_resultset));
                 cdao.setCategories(setString(i++, load_resultset));
+                String organizer = setString(i++, load_resultset);
+                if (organizer != null)
+                    cdao.setOrganizer(organizer);
+                cdao.setUid(setString(i++, load_resultset));
+                cdao.setSequence(setInt(i++, load_resultset));
                 cdao.setUsers(cimp.getUserParticipants(cdao, readcon, so.getUserId()).getUsers());
                 cdao.setParticipants(cimp.getParticipants(cdao, readcon).getList());
                 if (check_permissions && cdao.getEffectiveFolderId() != inFolder) {
@@ -219,6 +234,11 @@ public class CalendarOperation implements SearchIterator<CalendarDataObject> {
                         cdao.calculateRecurrence();
                     }
                 }
+                
+                ExternalUserParticipant[] externals = ParticipantStorage.getInstance().selectExternal(ctx, readcon, cdao.getObjectID());
+                cdao.setParticipants(ParticipantLogic.mergeFallback(cdao.getParticipants(), externals));
+                cdao.setConfirmations(ParticipantLogic.mergeConfirmations(externals, cdao.getParticipants()));
+                setAttachmentLastModified(readcon, ctx, cdao);
             } else {
                 final String text = "Object " + oid + " in context " + cdao.getContextID();
                 final OXObjectNotFoundException e = new OXObjectNotFoundException(OXObjectNotFoundException.Code.OBJECT_NOT_FOUND, APPOINTMENT, text);
@@ -489,7 +509,7 @@ public class CalendarOperation implements SearchIterator<CalendarDataObject> {
             }
             final UserParticipant up = new UserParticipant(uid);
             up.setConfirm(CalendarDataObject.ACCEPT);
-            recColl.checkAndConfirmIfUserUserIsParticipant(cdao, up);
+            recColl.checkAndConfirmIfUserUserIsParticipantInPublicFolder(cdao, up);
         }
 
         UserParticipant p = null;
@@ -690,6 +710,14 @@ public class CalendarOperation implements SearchIterator<CalendarDataObject> {
                                 final Participants participants = cimp.getParticipants(cdao, readcon);
                                 cdao.setParticipants(participants.getList());
                             }
+                        } else if (Appointment.CONFIRMATIONS == cols[a]) {
+                            if (CachedCalendarIterator.CACHED_ITERATOR_FAST_FETCH) {
+                                cdao.setFillConfirmations();
+                            } else {
+                                ExternalUserParticipant[] externals = ParticipantStorage.getInstance().selectExternal(cdao.getContext(), readcon, cdao.getObjectID());
+                                cdao.setParticipants(ParticipantLogic.mergeFallback(cdao.getParticipants(), externals));
+                                cdao.setConfirmations(ParticipantLogic.mergeConfirmations(externals, cdao.getParticipants()));
+                            }
                         } else if (Appointment.FOLDER_ID == cols[a]) {
                             if (recColl.getFieldName(Appointment.FOLDER_ID) != null) {
                                 if (oids == null) {
@@ -726,10 +754,17 @@ public class CalendarOperation implements SearchIterator<CalendarDataObject> {
                                     g++;
                                 }
                             }
+                        } else if (Appointment.LAST_MODIFIED_OF_NEWEST_ATTACHMENT == cols[a]) {
+                            if (CachedCalendarIterator.CACHED_ITERATOR_FAST_FETCH) {
+                                cdao.setFillLastModifiedOfNewestAttachment(true);
+                            } else {
+                                setAttachmentLastModified(readcon, c, cdao);
+                            }
                         } else {
                             throw new SearchIteratorException(
-                                    SearchIteratorException.SearchIteratorCode.NOT_IMPLEMENTED,
-                                    APPOINTMENT, Integer.valueOf(cols[a]));
+                                SearchIteratorException.SearchIteratorCode.NOT_IMPLEMENTED,
+                                APPOINTMENT,
+                                I(cols[a]));
                         }
                     } else {
                         ff.fillField(cdao, g++, co_rs);
@@ -812,7 +847,7 @@ public class CalendarOperation implements SearchIterator<CalendarDataObject> {
             if (cdao.getObjectID() == cdao.getRecurrenceID()) {
                 if (cdao.containsOccurrence() && !cdao.containsUntil()) {
                     // INFO: Somebody needs this value, have to check for side effects
-                    cdao.setUntil(new Date(recColl.normalizeLong((cdao.getStartDate().getTime() + (Constants.MILLI_DAY * cdao.getRecurrenceCalculator())))));
+                    //cdao.setUntil(new Date(recColl.normalizeLong((cdao.getStartDate().getTime() + (Constants.MILLI_DAY * cdao.getRecurrenceCalculator())))));
                 }
             } else {
                 // Recurring type on a change exception must be removed.
@@ -895,23 +930,21 @@ public class CalendarOperation implements SearchIterator<CalendarDataObject> {
     }
 
     static final Participant[] getNewParticipants(final Participant np[], final Participant op[]) {
-        final Participants p = new Participants();
-        for (int a = 0; a < np.length; a++ ) {
-            if (Arrays.binarySearch(op, np[a]) < 0) {
-                p.add(np[a]);
-            }
-        }
-        return p.getList();
+        return getNotContainedParticipants(np, op).getList();
     }
 
     static final Participant[] getDeletedParticipants(final Participant np[], final Participant op[]) {
+        return getNotContainedParticipants(np, op).getList();
+    }
+
+    static final Participants getNotContainedParticipants(Participant[] toCheck, Participant[] participants) {
         final Participants p = new Participants();
-        for (int a = 0; a < np.length; a++ ) {
-            if (Arrays.binarySearch(op, np[a]) < 0) {
-                p.add(np[a]);
+        for (Participant newParticipant : toCheck) {
+            if (Arrays.binarySearch(participants, newParticipant) < 0) {
+                p.add(newParticipant);
             }
         }
-        return p.getList();
+        return p;
     }
 
     /**
@@ -1179,6 +1212,9 @@ public class CalendarOperation implements SearchIterator<CalendarDataObject> {
             /*
              * An update of a change exception
              */
+            if (RecurrenceChecker.containsRecurrenceInformation(cdao)) {
+                throw new OXCalendarException(OXCalendarException.Code.CHANGE_EXCEPTION_TO_RECURRENCE);
+            }
             return recColl.RECURRING_NO_ACTION;
         }
         if (edao.containsRecurrenceType() && edao.getRecurrenceType() > CalendarDataObject.NO_RECURRENCE && (!cdao.containsRecurrenceType() || cdao.getRecurrenceType() == edao.getRecurrenceType())) {
@@ -1269,7 +1305,7 @@ public class CalendarOperation implements SearchIterator<CalendarDataObject> {
     private void calculateEndDateForNewType(final CalendarDataObject cdao, final CalendarDataObject edao) throws OXException {
         Date occurrenceDate;
         if (cdao.getOccurrence() <= 0) {
-            occurrenceDate = recColl.getOccurenceDate(cdao, recColl.MAXTC);
+            occurrenceDate = recColl.getOccurenceDate(cdao, recColl.MAX_OCCURRENCESE);
         } else {
             occurrenceDate = recColl.getOccurenceDate(cdao);
         }
@@ -1296,12 +1332,21 @@ public class CalendarOperation implements SearchIterator<CalendarDataObject> {
         // and then modify the recurring. Assume all data has been provided
         boolean pattern_change = false;
         boolean completenessChecked = false;
+        boolean changeStartDate = false;
 
         if (cdao.containsInterval() && cdao.getInterval() != edao.getInterval()) {
             recColl.checkRecurringCompleteness(cdao, !edao.containsUntil() && !edao.containsOccurrence());
             completenessChecked = true;
             pattern_change = true;
         }
+        if (cdao.containsStartDate() && !cdao.getStartDate().equals(edao.getStartDate())) {
+            if (!completenessChecked) {
+                recColl.checkRecurringCompleteness(cdao, !edao.containsUntil() && !edao.containsOccurrence());
+                completenessChecked = true;
+            }
+            pattern_change = true;
+            changeStartDate = true;
+        }
         if (cdao.containsDays() && cdao.getDays() != edao.getDays()) {
             if (!completenessChecked) {
                 recColl.checkRecurringCompleteness(cdao, !edao.containsUntil() && !edao.containsOccurrence());
@@ -1332,7 +1377,7 @@ public class CalendarOperation implements SearchIterator<CalendarDataObject> {
             // Calculate occurrence's time
             final Date occurrenceDate;
             if (cdao.getOccurrence() <= 0) {
-                occurrenceDate = recColl.getOccurenceDate(cdao, recColl.MAXTC);
+                occurrenceDate = recColl.getOccurenceDate(cdao, recColl.MAX_OCCURRENCESE);
             } else {
                 occurrenceDate = recColl.getOccurenceDate(cdao);
             }
@@ -1342,7 +1387,7 @@ public class CalendarOperation implements SearchIterator<CalendarDataObject> {
             cdao.setEndDate(calculateRealRecurringEndDate(untilDate, edao.getEndDate(), edao.getFullTime()));
             pattern_change = true;
         }
-        if (cdao.containsUntil() && edao.containsUntil() && recColl.check(cdao.getUntil(), edao.getUntil())) {
+        if (changeUntil(cdao, edao)) {
             if (!completenessChecked) {
                 recColl.checkRecurringCompleteness(cdao, !edao.containsUntil() && !edao.containsOccurrence());
                 completenessChecked = true;
@@ -1378,6 +1423,8 @@ public class CalendarOperation implements SearchIterator<CalendarDataObject> {
          */
         final int retval;
         if (pattern_change) {
+            if (!changeStartDate)
+                calculateAndSetRealRecurringStartAndEndDate(cdao, edao);
             cdao.setRecurrence(null);
 
             recColl.checkRecurring(cdao);
@@ -1400,6 +1447,16 @@ public class CalendarOperation implements SearchIterator<CalendarDataObject> {
         return retval;
     }
 
+    private static boolean changeUntil(final CalendarDataObject cdao, final CalendarDataObject edao) {
+        if (!cdao.containsUntil())
+            return false;
+        
+        if (cdao.containsUntil() && cdao.getUntil() == null && !edao.containsUntil())
+            return false;
+        
+        return cdao.containsUntil() && recColl.check(cdao.getUntil(), edao.getUntil());
+    }
+
     private static final void checkAndRemoveRecurrenceFields(final CalendarDataObject cdao) {
         if (cdao.containsDays()) {
             cdao.removeDays();
@@ -1434,6 +1491,21 @@ public class CalendarOperation implements SearchIterator<CalendarDataObject> {
             cdao.setShownAs(CalendarDataObject.RESERVED); // auto correction
         }
     }
+    
+    private void setAttachmentLastModified(Connection myCon, Context ctx, CalendarDataObject cdao) {
+        if (!cdao.containsObjectID())
+            return;
+        AttachmentBase attachmentBase = Attachments.getInstance(new SimpleDBProvider(myCon, null));
+        Date date = null;
+        try {
+            date = attachmentBase.getNewestCreationDate(ctx, Types.APPOINTMENT, cdao.getObjectID());
+        } catch (AttachmentException e) {
+            LOG.error(e.getMessage(), e);
+        }
+        if (null != date) {
+            cdao.setLastModifiedOfNewestAttachment(date);
+        }
+    }
 
     private static interface FieldFiller {
         public void fillField(CalendarDataObject cdao, int columnCount, ResultSet rs) throws SQLException;
@@ -1602,6 +1674,26 @@ public class CalendarOperation implements SearchIterator<CalendarDataObject> {
                     cdao.setRecurringStart(recurring_start);
                 }
             });
+            put(Integer.valueOf(Appointment.ORGANIZER), new FieldFiller() {
+                public void fillField(final CalendarDataObject cdao, final int columnCount, final ResultSet rs)
+                        throws SQLException {
+                    final String organizer = rs.getString(columnCount);
+                    cdao.setOrganizer(organizer);
+                }
+            });
+            put(Integer.valueOf(Appointment.UID), new FieldFiller() {
+                public void fillField(final CalendarDataObject cdao, final int columnCount, final ResultSet rs)
+                        throws SQLException {
+                    final String uid = rs.getString(columnCount);
+                    cdao.setUid(uid);
+                }
+            });
+            put(Integer.valueOf(Appointment.SEQUENCE), new FieldFiller() {
+                public void fillField(final CalendarDataObject cdao, final int columnCount, final ResultSet rs)
+                        throws SQLException {
+                    cdao.setSequence(rs.getInt(columnCount));
+                }
+            });
 
         }
     };
