@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -59,6 +59,7 @@ import java.util.HashMap;
 import java.util.Map;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+import org.omg.CORBA.CTX_RESTRICT_SCOPE;
 import com.openexchange.api.OXObjectNotFoundException;
 import com.openexchange.api.OXPermissionException;
 import com.openexchange.api2.AppointmentSQLInterface;
@@ -463,6 +464,7 @@ public class CalendarSql implements AppointmentSQLInterface {
     }
 
     public CalendarDataObject[] insertAppointmentObject(final CalendarDataObject cdao) throws OXException, OXPermissionException {
+        RecurrenceChecker.check(cdao);
         if (session == null) {
             throw new OXCalendarException(OXCalendarException.Code.ERROR_SESSIONOBJECT_IS_NULL);
         }
@@ -472,6 +474,8 @@ public class CalendarSql implements AppointmentSQLInterface {
         final UserConfiguration userConfig = Tools.getUserConfiguration(ctx, session.getUserId());
         try {
             final CalendarOperation co = new CalendarOperation();
+            if (cdao.containsRecurrenceType())
+                recColl.checkRecurring(cdao);
             if (co.prepareUpdateAction(cdao, null, session.getUserId(), cdao.getParentFolderID(), user.getTimeZone())) {
                 try {
                     final OXFolderAccess ofa = new OXFolderAccess(ctx);
@@ -567,6 +571,7 @@ public class CalendarSql implements AppointmentSQLInterface {
     }
 
     public CalendarDataObject[] updateAppointmentObject(final CalendarDataObject cdao, final int inFolder, final Date clientLastModified) throws OXException {
+        RecurrenceChecker.check(cdao);
         if (session == null) {
             throw new OXCalendarException(OXCalendarException.Code.ERROR_SESSIONOBJECT_IS_NULL);
         }
@@ -995,6 +1000,21 @@ public class CalendarSql implements AppointmentSQLInterface {
         }
         return cimp.setUserConfirmation(oid, folderId, uid, confirm, confirm_message, session, ctx);
     }
+    
+    public Date setExternalConfirmation(int oid, int folderId, String mail, int confirm, String message) throws OXException {
+        if (session == null) {
+            throw new OXCalendarException(OXCalendarException.Code.ERROR_SESSIONOBJECT_IS_NULL);
+        }
+        final Context ctx = Tools.getContext(session);
+        if (message != null) {
+            String error = null;
+            error = Check.containsInvalidChars(message);
+            if (error != null) {
+                throw new OXCalendarException(OXCalendarException.Code.INVALID_CHARACTER, "Confirm Message", error);
+            }
+        }
+        return cimp.setExternalConfirmation(oid, folderId, mail, confirm, message, session, ctx);
+    }
 
     public SearchIterator<Appointment> getObjectsById(final int[][] oids, int[] cols) throws OXException {
         if (session == null) {
@@ -1330,4 +1350,18 @@ public class CalendarSql implements AppointmentSQLInterface {
         return new StringBuilder(str.length() + 2).append('(').append(str).append(')').toString();
     }
 
+    /* (non-Javadoc)
+     * @see com.openexchange.api2.AppointmentSQLInterface#resolveUid(java.lang.String)
+     */
+    public int resolveUid(String uid) throws OXException {
+        return cimp.resolveUid(session, uid);
+    }
+
+    /* (non-Javadoc)
+     * @see com.openexchange.api2.AppointmentSQLInterface#getFolder(int)
+     */
+    public int getFolder(int objectId) throws OXException {
+        return cimp.getFolder(session, objectId);
+    }
+
 }
